//
//  EditSubChannelView.swift
//  SampleApp
//
//  Created by Amity on 7/11/2565 BE.
//  Copyright © 2565 BE David Zhang. All rights reserved.
//

import Foundation
import SwiftUI
import AmitySDK

struct EditSubChannelView: View {
    
    // Inject from other view
    @ObservedObject var viewModel: SubChannelManager
    
    @State var subChannelId: String
    @State var subChannelName: String
    
    // Alert
    @State private var isAlertShown = false
    @State private var alertMessage: String = ""
        
    var body: some View {
        VStack(alignment: .leading, spacing: 0) {
            TextField("Sub Channel Id", text: $subChannelId)
                .modifier(AmityTextFieldStyle())
                .keyboardType(.default)
                .autocapitalization(.none)
            
            Text("Use channel private id for testing")
                .font(.footnote)
                .foregroundColor(.secondary)
                .padding(.top, 4)
            
            TextField("Sub Channel Name", text: $subChannelName)
                .modifier(AmityTextFieldStyle())
                .padding(.top, 16)
                .accessibilityIdentifier("edit-sub-channel.sub-channel-name")
            
            AmityDefaultButton(title: "Update") {
                viewModel.updateSubChannel(displayName: subChannelName, subChannelId: subChannelId) { subChannel, error in
                    
                    if let error {
                        alertMessage = error.localizedDescription
                    } else {
                        alertMessage = "SubChannel Updated Successfully!"
                    }
                    
                    // Show alert
                    isAlertShown.toggle()
                }
            }
            .padding(.top, 16)
            
            AmityDefaultButton(title: "Soft Delete") {
                viewModel.deleteSubChannel(withId: subChannelId) { success, error in
                    
                    if let error {
                        alertMessage = error.localizedDescription
                    } else {
                        alertMessage = "SubChannel Deleted Successfully!"
                    }
                    
                    // Show Alert
                    isAlertShown.toggle()
                }
            }
            .padding(.top, 16)
            
            AmityDefaultButton(title: "Hard Delete") {
                viewModel.deleteSubChannel(withId: subChannelId, hardDelete: true) { success, error in
                    
                    if let error {
                        alertMessage = error.localizedDescription
                    } else {
                        alertMessage = "SubChannel Hard Deleted Successfully!"
                    }
                    
                    // Show Alert
                    isAlertShown.toggle()
                }
            }
            .padding(.top, 16)
            
            Spacer()
        }
        .navigationTitle("Edit SubChannel")
        .navigationBarTitleDisplayMode(.inline)
        .padding()
        .alert(isPresented: $isAlertShown) {
            Alert(title: Text(""), message: Text(alertMessage), dismissButton: .cancel(Text("Dismiss")))
        }
    }
}
