//
//  CommunityFeedViewModel.swift
//  SampleApp
//
//  Created by Sarawoot Khunsri on 1/7/2564 BE.
//  Copyright © 2564 BE David Zhang. All rights reserved.
//

import UIKit
import AmitySDK

class CommunityFeedViewModel: ObservableObject {
    
    private let feedRepository = AmityFeedRepository(client: AmityManager.shared.client!)
    private let postRepository =  AmityPostRepository(client: AmityManager.shared.client!)
    private let communityRepository = AmityCommunityRepository(client: AmityManager.shared.client!)
    
    private var token: AmityNotificationToken?
    private var postCollection: AmityCollection<AmityPost>?

    var community: CommunityModel
    
    var feed = [CommunityPostModel]()
    
    var publishedCount: Int {
        return getPostCount(by: .published)
    }
    
    var reviewingCount: Int {
        return getPostCount(by: .reviewing)
    }
    
    var isDataUpdated: Bool = false
    var feedType: AmityFeedType = .published
    init(community: CommunityModel) {
        self.community = community
        
    }
    
    func queryFeed(sort: AmityPostQuerySortOption, feedType: AmityFeedType, completion: (() -> Void)?) {
        self.feedType = feedType
        if !feed.isEmpty {
            feed = []
            completion?()
        }
        isDataUpdated = false
        postCollection = feedRepository.getCommunityFeed(withCommunityId: community.id, sortBy: sort, includeDeleted: false, feedType: feedType)
        token = postCollection?.observe({ [weak self] (collection, _, _) in
            guard collection.dataStatus == .fresh, let strongSelf = self else { return }
            
//            self?.token?.invalidate()
            Log.add(info: "Feed observed: \(collection.dataStatus.description), post count: \(collection.count())")
            
            var list = [CommunityPostModel]()
            for post in collection.allObjects() {
                let model = CommunityPostModel(postId: post.postId,
                                               postedUserDisplayName: post.postedUser?.displayName,
                                               text: post.data?["text"] as? String,
                                               feedType: post.getFeedType(),
                                               isDeleted: post.isDeleted,
                                               postDataType: post.dataType,
                                               isOwner: post.postedUserId == AmityManager.shared.client?.currentUserId,
                                               createdAt: post.createdAt,
                                               metadata: post.metadata,
                                               mentionees: post.mentionees)
                list.append(model)
            }
            strongSelf.isDataUpdated = true
            strongSelf.feed = list
            completion?()
        })
    }
    
    func post(at indexPath: IndexPath) -> CommunityPostModel {
        return feed[indexPath.row]
    }
    
    func getPostCount(by feedType: AmityFeedType) -> Int {
        return community.getCommunityObject().getPostCount(feedType: feedType)
    }
    
    func approve(post: CommunityPostModel, completion: ((String) -> Void)?) {
        Task { @MainActor in
            do {
                let result = try await postRepository.approvePost(withId: post.postId)
                completion?("Success! Approved post")
            } catch {
                completion?("Something wrong!, \(error)")
            }
        }
    }
    
    func decline(post: CommunityPostModel, completion: ((String) -> Void)?) {
        Task { @MainActor in
            do {
                let result = try await postRepository.declinePost(withId: post.postId)
                completion?("Success! Decline post")
            } catch {
                completion?("Something wrong!, \(error)")
            }
        }
    }
    
    func delete(post: CommunityPostModel, completion: ((String) -> Void)?) {
        Task { @MainActor in
            if post.isDeleted {
                completion?("Sorry! Post is already deleted")
            } else {
                do {
                    let _ = try await postRepository.softDeletePost(withId: post.postId, parentId: nil)
                    completion?("Success! delete post")
                } catch {
                    completion?("Something wrong!, \(error)")
                }
            }
        }
    }
    
}
